
# q-expansion of newform 1920.2.w.c, downloaded from the LMFDB on 05 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1920
weight = 2
poly_data = [1, 0, 0, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[511, [-1, 0, 0, 0]], [901, [1, 0, 0, 0]], [641, [1, 0, 0, 0]], [1537, [0, 0, 1, 0]]]
aps_data = [[0, 0, 0, 0], [0, -1, 0, 0], [2, 0, -1, 0], [-2, 0, 2, 0], [0, 2, -2, 2], [3, 0, -3, 2], [1, -2, 1, 0], [0, 0, 0, 0], [-2, 4, -2, 0], [0, -4, 0, -4], [0, 2, -6, 2], [1, 10, 1, 0], [4, -2, 0, 2], [6, 4, 6, 0], [0, 0, 0, -4], [3, 0, -3, -4], [6, -2, 0, 2], [0, -6, 0, 6], [2, 0, -2, 4], [0, -4, -8, -4], [1, 0, -1, -12], [2, -2, 0, 2], [8, 4, 8, 0], [0, 6, -4, 6], [5, -8, 5, 0], [-2, 8, 0, -8], [-4, -12, -4, 0], [-2, 0, 2, -16], [0, 4, -10, 4], [-3, 0, 3, 10], [0, 0, 0, -4], [0, -2, 10, -2], [3, 14, 3, 0], [0, -8, 0, 8], [0, -8, -10, -8], [0, -2, -10, -2], [7, -6, 7, 0], [-4, -12, -4, 0], [-4, 0, 4, 4], [-5, 0, 5, 0], [-2, 14, 0, -14], [2, 4, 0, -4], [0, 0, 4, 0], [3, 0, -3, 4], [11, 4, 11, 0], [-6, -2, 0, 2], [0, -4, 0, -4], [0, -20, 0, 0], [14, 0, -14, 0], [0, -6, 0, -6], [1, 0, -1, -2], [8, 4, 0, -4], [-12, -8, 0, 8], [0, -2, 2, -2], [1, 6, 1, 0], [4, 12, 4, 0], [0, 0, 6, 0], [0, 2, 18, 2], [-1, 26, -1, 0], [-28, -2, 0, 2], [14, -12, 14, 0], [-15, 0, 15, -4], [12, 0, -12, -4], [0, 8, 0, 8], [-17, 0, 17, 4], [-11, -12, -11, 0], [0, 16, 8, 16], [-3, 20, -3, 0], [2, 0, -2, 16], [0, 10, 8, 10], [-13, 0, 13, 10], [12, 12, 0, -12], [-18, 0, 18, 0], [-7, 0, 7, 2], [-24, -4, 0, 4], [-6, 12, -6, 0], [0, -16, 6, -16], [13, -6, 13, 0], [-22, 8, 0, -8], [0, 0, 22, 0], [-10, -10, 0, 10], [-8, -6, 0, 6], [0, -16, -8, -16], [-11, 0, 11, 4], [2, -18, 0, 18], [-6, -8, -6, 0], [0, -8, 6, -8], [-5, -16, -5, 0], [-14, 0, 0, 0], [10, 0, 10, 0], [16, 0, -16, -20], [4, 8, 0, -8], [-12, 0, 12, -12], [0, -10, -14, -10], [-8, 12, 0, -12], [16, -12, 16, 0], [0, 12, -4, 12], [-18, -8, 0, 8], [8, -12, 8, 0], [-2, -12, 0, 12], [-6, 0, 6, -4], [15, -8, 15, 0], [2, 24, 2, 0], [0, 8, -26, 8], [0, 0, 4, 0], [11, 8, 11, 0], [6, 0, -6, 16], [-5, 0, 5, -18], [8, -4, 0, 4], [24, -8, 0, 8], [-10, 0, 10, -16], [-13, 0, 13, -6], [21, 2, 21, 0], [36, 8, 0, -8], [0, 22, -18, 22], [12, 10, 0, -10], [-4, -36, -4, 0], [-16, 0, 16, 12], [3, 0, -3, -8], [-2, 22, 0, -22], [0, -22, 0, 22], [1, 0, -1, 28], [23, -8, 23, 0], [0, 12, 0, 0], [0, 28, -8, 28], [-4, 20, 0, -20], [0, -10, -32, -10], [-24, 12, 0, -12], [12, 0, -12, -4], [21, 0, -21, 2], [0, -12, 0, 12], [-14, 20, -14, 0], [0, -26, -10, -26], [7, -22, 7, 0], [10, -16, 0, 16], [0, -12, 24, -12], [-3, 0, 3, 12], [-4, 0, 4, -36], [13, 8, 13, 0], [0, 6, 12, 6], [0, 16, 32, 16], [16, -4, 0, 4], [-24, -4, -24, 0], [-24, 0, 24, -12], [0, 4, -30, 4], [24, -4, 0, 4], [-1, 0, 1, -6], [-21, 2, -21, 0], [20, -12, 0, 12], [0, -36, 0, 0], [-9, 2, -9, 0], [4, 10, 0, -10], [24, -12, 24, 0], [-20, 0, 20, 12], [-2, 0, 2, 4], [0, 4, -12, 4], [-2, -22, 0, 22], [0, -32, 2, -32], [-21, -24, -21, 0], [2, 0, 0, 0], [-12, 0, 12, 4], [13, 0, -13, -18], [20, 0, -20, -4], [0, 6, -14, 6], [3, 26, 3, 0], [-30, -12, -30, 0], [0, 10, 10, 10], [15, 18, 15, 0], [0, 16, -32, 16], [5, 0, -5, 8], [38, -10, 0, 10], [10, -20, 0, 20], [0, -12, 4, -12], [9, 0, -9, 36], [-14, -2, 0, 2], [0, -30, 4, -30], [0, 20, 16, 20], [38, 16, 0, -16], [10, -32, 10, 0], [0, -20, 18, -20], [-16, 0, 16, 12], [0, 2, 14, 2], [13, 0, -13, 18], [-1, -54, -1, 0], [10, 44, 10, 0], [0, -8, 28, -8], [1, 10, 1, 0], [-10, -28, -10, 0], [0, 16, -38, 16], [0, 8, 48, 8], [-13, 0, 13, 12], [0, -20, 0, 0], [0, 12, -32, 12], [8, -16, 0, 16], [22, 0, -22, 16], [13, 0, -13, -30], [4, 40, 0, -40], [17, 0, -17, -34], [-5, 26, -5, 0], [-38, -4, -38, 0], [0, 16, 12, 16], [0, 26, -30, 26], [-25, 6, -25, 0], [0, 28, 4, 28], [-46, -14, 0, 14], [-31, -20, -31, 0], [2, -34, 0, 34], [14, -32, 14, 0], [0, 2, 28, 2], [0, -28, 28, -28], [-17, -28, -17, 0], [48, -16, 0, 16], [16, 4, 16, 0], [-36, 0, 36, -4], [-24, -16, 0, 16], [-40, 12, 0, -12], [10, 0, -10, -56], [6, -8, 0, 8], [30, 0, -30, 4], [-7, 0, 7, 16], [-32, -10, 0, 10], [-10, 10, 0, -10], [0, -30, 4, -30], [-14, 24, -14, 0], [-12, 0, 12, -4], [0, -20, -18, -20], [-5, 0, 5, -26], [-36, -20, 0, 20], [18, 0, -18, 8], [0, -10, -54, -10], [-33, 0, 33, 10], [40, 4, 0, -4], [0, 34, -14, 34], [12, -14, 0, 14], [-16, 20, -16, 0], [-6, 0, 6, 36], [0, 16, -14, 16], [-17, 0, 17, 16], [42, 2, 0, -2], [0, -36, 0, -36], [-26, -24, -26, 0], [0, 32, -12, 32], [8, -4, 8, 0], [0, 18, -24, 18], [-5, 0, 5, 26], [-20, 8, 0, -8], [-2, 0, 2, -64], [0, -22, -10, -22], [16, 4, 0, -4], [44, 12, 44, 0], [5, -50, 5, 0], [2, -16, 0, 16], [-6, 0, 6, 20], [0, -16, -18, -16], [-5, 0, 5, -20], [30, -34, 0, 34], [-2, -12, 0, 12], [26, 0, -26, -4], [1, -28, 1, 0], [15, 4, 15, 0], [0, 4, 0, 0], [-12, 0, 12, -12], [0, -14, 0, -14], [-5, 0, 5, 18], [-23, -14, -23, 0], [24, -28, 0, 28], [0, -24, 28, -24], [30, 8, 0, -8], [-44, -4, -44, 0], [-15, 0, 15, -16], [18, -44, 0, 44], [-44, 0, 44, -20], [-5, 0, 5, -44], [-2, -6, 0, 6], [-17, 4, -17, 0], [32, 12, 32, 0], [-14, 0, 14, 8], [0, 14, 48, 14], [4, 0, 0, 0], [0, -10, -38, -10], [38, -12, 38, 0], [0, -30, 34, -30], [-28, 0, 28, 28], [26, 20, 0, -20], [30, 0, -30, 4], [0, -16, 52, -16], [-9, 0, 9, 32], [-1, 72, -1, 0], [14, 10, 0, -10], [0, 40, 2, 40], [-12, -8, 0, 8], [38, 0, -38, 8], [-17, 0, 17, -6], [0, -26, -22, -26], [13, 0, -13, -26], [0, -8, -46, -8], [0, 34, 18, 34], [-15, 0, 15, -36], [14, -26, 0, 26], [-34, 0, 34, 12], [9, 0, -9, -4], [9, -56, 9, 0], [10, 38, 0, -38]]
